package com.laguerre.kan;

import android.view.MotionEvent;

public final class Input {
	private static Input instance = null;

	public static Input getInstance() {
		if (instance == null) {
			instance = new Input();
		}
		return instance;
	}

	private float xyFullStep = 20f;
	// Some "down" actions seem to be "lost" whilst the "up" actions seem to
	// work pretty well... So by this boolean, we'll check whether a "down"
	// has been processed, otherwise we'll initialize "previousX" and "previousY"
	// with the current touch info.
	private boolean prevPosToReset = true;
    private float previousX = 0f;
    private float previousY = 0f;

	public void onTouchEvent(MotionEvent e) {
	    float x = e.getX();
	    float y = e.getY();

	    switch (e.getActionMasked()) {
	    case MotionEvent.ACTION_DOWN: {
	    	// Reset the previous position when a new movement begin.
	    	this.previousX = x;
	    	this.previousY = y;
	    	// Ok, we did the reset of previous positions.
	    	this.prevPosToReset = false;
	    }
	    break;
	    case MotionEvent.ACTION_MOVE: {
	    	if (this.prevPosToReset) {
		    	this.previousX = x;
		    	this.previousY = y;
		    	this.prevPosToReset = false;
		    	// Discard this process because we will not translate at all.
		    	break;
	    	}

	    	float dx = x - this.previousX;
	    	float dy = y - this.previousY;

	    	if (Math.abs(dx) > 2f) {
	    		dx = Math.signum(dx) * 2f;
	    	}
	    	if (Math.abs(dy) > 2f) {
	    		dy = Math.signum(dy) * 2f;
	    	}

	    	final float xNormalized = dx / SurfaceInfo.getInstance().getSurfaceWidth();
	    	final float yNormalized = dy / SurfaceInfo.getInstance().getSurfaceHeight();

	    	final float xMove = xNormalized * this.xyFullStep;
	    	final float yMove = yNormalized * this.xyFullStep;

	    	Camera.getCurrent().getSharedTransform().translateLocally(xMove, -yMove, 0f);

	    	this.previousX = x;
	    	this.previousY = y;
	    }
	    break;
	    case MotionEvent.ACTION_CANCEL:
	    case MotionEvent.ACTION_UP:
	    	this.prevPosToReset = true;
	    	break;
	    default:
	    	break;
	    }
	}
}
