package com.laguerre.mytest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import com.laguerre.kan.ManualMesh;

public class SimpleQuad extends ManualMesh {
	private static final float[] quadVertices = {
		-1.0f,  1.0f, 0.0f, 1.0f,   // top left
		-1.0f, -1.0f, 0.0f, 1.0f,   // bottom left
		 1.0f, -1.0f, 0.0f, 1.0f,   // bottom right
		 1.0f,  1.0f, 0.0f, 1.0f    // top right
	};

    private static final float[] quadNormals = {
        0.0f, 0.0f, 1.0f,   // top left
        0.0f, 0.0f, 1.0f,   // bottom left
        0.0f, 0.0f, 1.0f,   // bottom right
        0.0f, 0.0f, 1.0f    // top right
    };

	private static final float[] quadUV = {
		0.0f, 1.0f, // top left
		0.0f, 0.0f, // bottom left
		1.0f, 0.0f, // bottom right
		1.0f, 1.0f  // top right
	};

	private static short[] drawList = {
		0, 1, 2, 0, 2, 3
	};

	private FloatBuffer packedDataBuffer;
	private ShortBuffer drawListBuffer;

    @Override
    public int getNbOfSubMeshes() {
    	return 1;
    }

	@Override
	public void build() {
    	final int vertexCount = quadVertices.length / POSITION_DATA_SIZE;
    	final int dataSize = getVertexPackedDataSize(0);

    	final float[] vertexData = new float[vertexCount * dataSize];
    	for (int i = 0; i < vertexCount; i++) {
    		final int start = i * dataSize;
    		// Position
    		final int readPositionStart = POSITION_DATA_SIZE * i;
    		vertexData[start    ] = quadVertices[readPositionStart    ];  // x
    		vertexData[start + 1] = quadVertices[readPositionStart + 1];  // y
    		vertexData[start + 2] = quadVertices[readPositionStart + 2];  // z
    		vertexData[start + 3] = quadVertices[readPositionStart + 3];  // w
    		// Normal
    		final int readNormalStart = NORMAL_DATA_SIZE * i;
    		vertexData[start + 4] = quadNormals[readNormalStart    ];  // x
    		vertexData[start + 5] = quadNormals[readNormalStart + 1];  // y
    		vertexData[start + 6] = quadNormals[readNormalStart + 2];  // z
    		// UV
    		vertexData[start + 7] = quadUV[UV_DATA_SIZE * i    ];  // u
    		vertexData[start + 8] = quadUV[UV_DATA_SIZE * i + 1];  // v
    	}

    	final ByteBuffer bb = ByteBuffer.allocateDirect(vertexCount * dataSize * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.packedDataBuffer = bb.asFloatBuffer();
    	this.packedDataBuffer.put(vertexData);
    	this.packedDataBuffer.position(0);

		final ByteBuffer bbDrawList = ByteBuffer.allocateDirect(drawList.length * Short.SIZE / 8);
    	bbDrawList.order(ByteOrder.nativeOrder());
    	this.drawListBuffer = bbDrawList.asShortBuffer();
    	this.drawListBuffer.put(drawList);
    	this.drawListBuffer.position(0);
	}

	@Override
	public FloatBuffer getSharedVertexPackedData(int subMeshId) {
		return this.packedDataBuffer;
	}

	@Override
	public int getVertexCount(int subMeshId) {
		return 4;
	}

	@Override
	public int getDrawListCount(int subMeshId) {
		return drawList.length;
	}

	@Override
	public ShortBuffer getSharedDrawListBuffer(int subMeshId) {
		return this.drawListBuffer;
	}
}
