package com.laguerre.mytest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import com.laguerre.kan.ManualMesh;

public class SimpleTriangle extends ManualMesh {
	private ShortBuffer drawListBuffer;

    private static final float[] triVertices = {
         0.0f,  0.622008459f, 0.0f,   // top
        -0.5f, -0.311004243f, 0.0f,   // bottom left
         0.5f, -0.311004243f, 0.0f    // bottom right
    };
    private static final float[] triNormals = {
        0.0f, 0.0f, 1.0f, 1.0f,   // top
        0.0f, 0.0f, 1.0f, 1.0f,   // bottom left
        0.0f, 0.0f, 1.0f, 1.0f    // bottom right
   };
    private static final float[] triUV = {
    	0.5f, 1.0f,
    	0.0f, 0.0f,
    	1.0f, 0.0f
    };
    private static short[] drawList = {
		0, 1, 2
    };

    private FloatBuffer packedData;

    @Override
    public int getNbOfSubMeshes() {
    	return 1;
    }

	@Override
	public void build() {
    	final int vertexCount = triVertices.length / POSITION_DATA_SIZE;
    	final int dataSize = getVertexPackedDataSize(0);

    	final float[] vertexData = new float[vertexCount * dataSize];
    	for (int i = 0; i < vertexCount; i++) {
    		final int start = i * dataSize;
    		// Position
    		final int readPositionStart = POSITION_DATA_SIZE * i;
    		vertexData[start    ] = triVertices[readPositionStart    ];  // x
    		vertexData[start + 1] = triVertices[readPositionStart + 1];  // y
    		vertexData[start + 2] = triVertices[readPositionStart + 2];  // z
    		vertexData[start + 3] = triVertices[readPositionStart + 3];  // w
    		// Normal
    		final int readNormalStart = NORMAL_DATA_SIZE * i;
    		vertexData[start + 4] = triNormals[readNormalStart    ];  // x
    		vertexData[start + 5] = triNormals[readNormalStart + 1];  // y
    		vertexData[start + 6] = triNormals[readNormalStart + 2];  // z
    		// UV
    		vertexData[start + 7] = triUV[UV_DATA_SIZE * i    ];  // u
    		vertexData[start + 8] = triUV[UV_DATA_SIZE * i + 1];  // v
    	}

    	final ByteBuffer bb = ByteBuffer.allocateDirect(vertexCount * dataSize * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.packedData = bb.asFloatBuffer();
    	this.packedData.put(vertexData);
    	this.packedData.position(0);

        final ByteBuffer bbDrawList = ByteBuffer.allocateDirect(drawList.length * Short.SIZE / 8);
        bbDrawList.order(ByteOrder.nativeOrder());
        this.drawListBuffer = bbDrawList.asShortBuffer();
    	this.drawListBuffer.put(drawList);
    	this.drawListBuffer.position(0);
	}

	@Override
	public FloatBuffer getSharedVertexPackedData(int subMeshId) {
		return this.packedData;
	}

	@Override
	public final int getVertexCount(int subMeshId) {
		return 3;
	}

	@Override
	public int getDrawListCount(int subMeshId) {
		return 3;
	}

	@Override
	public ShortBuffer getSharedDrawListBuffer(int subMeshId) {
		return this.drawListBuffer;
	}
}
